@@ -25,16 +25,17 @@ extern UV valuation(UV n, UV k);
 extern UV logint(UV n, UV b);
 extern UV mpu_popcount_string(const char* ptr, uint32_t len);
 
-extern signed char* _moebius_range(UV low, UV high);
-extern UV*    _totient_range(UV low, UV high);
+extern signed char* range_moebius(UV low, UV high);
+extern UV*    range_totient(UV low, UV high);
 extern IV     mertens(UV n);
-extern long double chebyshev_function(UV n, int which); /* 0 = theta, 1 = psi */
+extern NV chebyshev_psi(UV n);
+extern NV chebyshev_theta(UV n);
 
-extern long double Ei(long double x);
-extern long double Li(long double x);
+extern NV Ei(NV x);
+extern NV Li(NV x);
 extern long double ld_riemann_zeta(long double x);
 extern long double RiemannR(long double x);
-extern long double lambertw(long double k);
+extern NV lambertw(NV k);
 extern UV inverse_li(UV x);
 extern UV inverse_R(UV x);
 
